//
// GameX
// Ship Demo - Ship Class Code
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#include <math.h>

#include "shipdemox-ship.hpp"

// Ship - Initialize
// Initialize all ship images, ship shadow and ship's target image
void Ship::Initialize (int num)
{
	char tiffile[100];
	ImageExt imgext;
	int n;

	ship_num = num;									// Initialize ship number													

	if (ship_num==0) {
        photon_snd.LoadWav ("torpedo.wav");		
	} else {
		photon_snd.LoadWav ("torpedo.wav");
	}	
	ship_rotated.Size (80, 80, IMG_HIGHCOLOR | IMG_ALPHA);	// Initialize rotation image
	
	for (n=SHIP_IMG_LEFT; n<=SHIP_IMG_RIGHT; n++) {	// Load all ship images
		if (ship_num==0) {							// If ship number is 0...
			sprintf (tiffile, "Images\\ship-red%04d.tif", n);	// Load red ships.
		} else {									// Otherwise (ship number is 1)...
			sprintf (tiffile, "Images\\ship-blue%04d.tif", n);	// Load blue ships.
		}
		imgext.Load (tiffile, &ships[n], DRAW_MASKED);	// Load image (prepare for masked drawing)
	}
													// Load ship's shadow image
	imgext.Load ("Images\\ship-shadow.tif", &ships[SHIP_IMG_SHADOW], DRAW_ALPHA);

													// Load ship's target image
	imgext.Load ("Images\\target.tif", &target, DRAW_MASKED);	

	
	curr_img = SHIP_IMG_FLAT;						// Set current ship image
	ang = 0;										// Set initial ship angle
	refire = 0;	
	y = 100;										// Set initial ship position
	if (ship_num==0) 	x = 200;	
	else				x = 400;
}

// Ship - Move
// Get user input and move the ship
void Ship::Move (void)
{
	if (ship_num==0) {							// Process user input for Red Ship (num 0)...
		if (GameX.GetKeyDown(VK_LEFT)) {			// Left Key Pressed
			dang += 1; curr_img++;					// Rotate ship left
			if (curr_img>16) curr_img = 16;
		} else if (GameX.GetKeyDown(VK_RIGHT)) {	// Right Key Pressed
			dang -= 1; curr_img--;					// Rotate ship right
			if (curr_img<0) curr_img = 0;
		} else {									// Otherwise..
			if (dang>0) dang /= 2;					// Gradually stop the ship from
			if (dang<0) dang /= 2;					// rotating
			if (curr_img<9) curr_img++;
			if (curr_img>9) curr_img--;
		}
		if (GameX.GetKeyDown(VK_UP)) {				// Up Key Pressed
			vx += (float) cos(ang*DEGtoRAD)*.4f;	// Increase ship velocity
			vy -= (float)sin(ang*DEGtoRAD)*.4f;		
		} else if (GameX.GetKeyDown(VK_DOWN)) {		// Down Key Pressed
			vx += - (float) cos(ang*DEGtoRAD)*.4f;	// Decrease ship velocity
			vy -= - (float) sin(ang*DEGtoRAD)*.4f;
		} else {									// Otherwise..
			vx *= 0.9f;								// Gradually bring ship to a stop
			vy *= 0.9f;
		}
		if (GameX.GetKeyDown(VK_SPACE)) {			// Space Key Pressed
			if (refire++ > 2) {						// If refire counter is ready..
				refire = 0;							// Determine direction & velocity of 
				float fire_x, fire_y;				// the new 'fired' photon
				fire_x = vx + (float) cos(ang*DEGtoRAD) * 20.0f;
				fire_y = vy - (float) sin(ang*DEGtoRAD) * 20.0f;
							// Add a photon to the currently assigned Photon class
				use_photons->AddPhoton (x, y, fire_x, fire_y);
				GameX.PlaySound (&photon_snd, PLAY_REWIND);
			}
		} 	
	} else {									// Process user input for Blue Ship (num=1)
		if (GameX.GetKeyDown('A')) {				
			dang += 1; curr_img++;					// Code is identical to that for Red Ship
			if (curr_img>16) curr_img = 16;			// except that different keyboard
		} else if (GameX.GetKeyDown('S')) {			// keys are checked.
			dang -= 1; curr_img--;
			if (curr_img<0) curr_img = 0;
		} else {		
			if (dang>0) dang /= 2;
			if (dang<0) dang /= 2;
			if (curr_img<9) curr_img++;
			if (curr_img>9) curr_img--;
		}
		if (GameX.GetKeyDown('W')) {			
			vx += (float) cos(ang*DEGtoRAD)*.4f;
			vy -= (float) sin(ang*DEGtoRAD)*.4f;		
		} else if (GameX.GetKeyDown('Z')) {		
			vx += - (float) cos(ang*DEGtoRAD)*.4f;
			vy -= - (float) sin(ang*DEGtoRAD)*.4f;
		} else {	
			vx *= 0.9f;
			vy *= 0.9f;
		}
		if (GameX.GetKeyDown('X')) {
			if (refire++ > 2) {
				refire = 0;
				float fire_x, fire_y;
				fire_x = vx + (float) cos(ang*DEGtoRAD) * 20.0f;
				fire_y = vy - (float) sin(ang*DEGtoRAD) * 20.0f;
				use_photons->AddPhoton (x, y, fire_x, fire_y);
				GameX.PlaySound (&photon_snd, PLAY_REWIND);
			}
		} 	
	}
													//*** Move the ship
	ang += dang;									// Adjust angle by change-in-angle	
	if (ang<0) ang += 360;
	if (ang>360) ang -= 360;	
	x += vx;										// Move ship position by ship's velocity
	y += vy;
}

// Ship - Collisions
// Do ship collision detection with another ship
void Ship::Collisions (Ship* ship2)
{
	float dist;
	float speed, speed2;
	float vx2, vy2;
	
	target_x = ship2->GetX() - x;		// Calculate vector to 2nd ship
	target_y = ship2->GetY() - y;
										// Determine distance to 2nd ship
	dist = (float) sqrt(target_x*target_x + target_y*target_y);
	target_x /= dist;					// Determine direction to 2nd ship
	target_y /= dist;					// (is normalized vector)
	
	if (dist<40) {						// If distance to 2nd ship is less than 40 pixels..
		if (ship_num==0) {				// Do collision only if 'this' ship is Red.
		speed = (float) sqrt(vx*vx + vy*vy);		// Calculate speed of this ship (Red)
		vx2 = ship2->GetVX(); vy2 = ship2->GetVY();	
		speed2 = (float) sqrt(vx2*vx2 + vy2*vy2);	// Calculate speed of 2nd ship (Blue)
			// Calculate 'rebound' vector to throw ships in opposite directions 
			// (with speed equal to the sum of speeds of the two ships)								
		vx2 = -target_x * (speed+speed2)*0.25f;		
		vy2 = -target_y * (speed+speed2)*0.25f;
		vx += vx2;						// Add rebound to the Red ship's velocity
		vy -= vy2;
		ship2->AddVX(-vx2);				// Add rebound to the Blue ship's velocity
		ship2->AddVY(-vy2);
		}
	}
	target_x *= 100;					// Move target 100 pixels away from ship
	target_y *= 100;					// (so that we can draw it)

	if (x<0) vx = -vx;					// If ship rebounds off the view, bring
	if (x>800) vx = -vx;				// it back into view.
	if (y<0) vy = -vy;
	if (y>400) vy = -vy;
}

// Ship - Draw
// Draw the ship's shadow, target, and the ship itself
void Ship::Draw (int frame)
{
	// Rotate ships shadow by angle of ship
	ships[SHIP_IMG_SHADOW].Rotate (&ship_rotated, (float) ang-90, 1.0f);
	// Draw ships shadow using GamX
	GameX.DrawAlpha ((int) x-40, (int) y-40, &ship_rotated);

	// Rotate ship itself by angle of ship
	ships[curr_img].Rotate (&ship_rotated, (float) ang-90, 1.0f);
	// Draw ship using GameX
	GameX.DrawMasked ((int) x-40, (int) y-40, &ship_rotated);

	// Draw ship's target (not rotated)
	GameX.DrawMasked ((int) x + (int) target_x - 20, (int) y + (int) target_y-20, &target);
}