//
// GameX
// Ship Demo - Ship Class Header
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 


#ifndef SHIPDEMO_SHIP
	#define SHIPDEMO_SHIP

	#define GAMEX_WINDOWS
	#include "gamex.hpp"

	#include "shipdemox-photons.hpp"

	// The Ships are implemented as simple Finite-State-Machines

											//*** Ship Images
	#define SHIP_IMG_LEFT			0
	#define SHIP_IMG_FLAT			8
	#define SHIP_IMG_RIGHT			16
	#define SHIP_IMG_SHADOW			17
	#define SHIP_IMG_SKID			18
											//*** Ship States
	#define SHIP_STATE_NORMAL		0		// Normal Flight
	#define SHIP_STATE_BANKLEFT		1		// Banking Left
	#define SHIP_STATE_BANKRIGHT	2		// Banding Right

	class Ship {							//*** Ship class
	public:
		void Initialize (int num);			// Initialize ship number 'num'
		void Move (void);					// Move the ship
		void Draw (int frame);				// Draw the ship
		
		void Collisions (Ship* ship2);		// Collision Detection with another ship
		void UsePhotons (Photons* p) {use_photons = p;}	// Assign a Photon class to the ship

		float GetX (void) {return x;}		// Get position and velocity of ship
		float GetY (void) {return y;}
		float GetVX (void) {return vx;}
		float GetVY (void) {return vy;}
		void AddVX (float ax) {vx += ax;}
		void AddVY (float ay) {vy += ay;}
	private:
		int			ship_num;				// Current ship number
		int			state;					// Current ship state		
		
		float		ang, dang;				// Angle of ship (and change in angle)
		float		x, y;					// Position of ship
		float		vx, vy;					// Velocity of ship
		int			refire;					// Refire counter for ship

		int			curr_img;				// Current ship image
		float		target_x, target_y;		// Position of ship's target
		
		ImageX		ships[30];				// Ship images (TIF)
		ImageX		ship_rotated;			// Image of rotated ship (for display)
		ImageX		target;					// Ship's Target image (TIF)

		Photons*	use_photons;			// Photon class to handle 'fire' commands.
		
		SoundX		photon_snd;				// Sound of photon firing!
	};

#endif